package org.foxarmy.barcodescannerforemployees.activities

import android.app.DatePickerDialog
import android.content.DialogInterface
import android.content.Intent
import android.os.Bundle
import android.view.View
import android.widget.*
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import com.google.mlkit.vision.codescanner.GmsBarcodeScanning
import org.foxarmy.barcodescannerforemployees.DBStorageController
import org.foxarmy.barcodescannerforemployees.R
import org.foxarmy.barcodescannerforemployees.dataclasses.AbstractProduct
import org.foxarmy.barcodescannerforemployees.dataclasses.Product
import org.foxarmy.barcodescannerforemployees.views.AbstractProductView
import java.util.*

class AddProductActivity : AppCompatActivity() {

    private lateinit var scanButton: Button
    private lateinit var abstractProductView: AbstractProductView
    private lateinit var expiryDateRadioButton: RadioButton
    private lateinit var shelfLifeRadioButton: RadioButton

    private lateinit var expiryDateTextView: TextView
    private lateinit var expiryDateSelectButton: Button

    private lateinit var shelfLifeTextView: TextView
    private lateinit var shelfLifeTextEdit: EditText

    private lateinit var amountTextEdit: EditText

    private lateinit var dateOfProductionSelectButton: Button
    private lateinit var saveProductButton: Button

    private var expiryDateOverShelfLife: Boolean? = null
    private var dateOfProduction: String = ""
    private var dateOfExpiry: String = ""

    private var barcode: String = ""

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        setContentView(R.layout.fragment_add_product)

        val extras = intent.extras
        var product = extras!!.get("product") as Product?


        scanButton = findViewById(R.id.scanButton)
        abstractProductView = findViewById(R.id.abstractProductView)
        expiryDateRadioButton = findViewById(R.id.expiryDateRadio)
        shelfLifeRadioButton = findViewById(R.id.shelfLifeRadio)

        expiryDateTextView = findViewById(R.id.expiryDateTextView)
        expiryDateSelectButton = findViewById(R.id.selectExpiryDateButton)

        shelfLifeTextView = findViewById(R.id.shelfLife)
        shelfLifeTextEdit = findViewById(R.id.shelfLifeTextEdit)

        amountTextEdit = findViewById(R.id.amountTextEdit)

        dateOfProductionSelectButton = findViewById(R.id.selectDateOfProductionButton)
        saveProductButton = findViewById(R.id.saveProductButton)

        scanButton.setOnClickListener {
            val scanner = GmsBarcodeScanning.getClient(this)
            scanner.startScan()
                .addOnSuccessListener { bc ->
                    barcode = bc.rawValue.toString()
                    findAndDisplayAbstractProductByBarcode(barcode)
                }
                .addOnFailureListener {
                    Toast.makeText(this, "Cannot scan barcode", Toast.LENGTH_SHORT).show()
                }
        }

        expiryDateRadioButton.setOnClickListener {
            expiryDateTextView.visibility = View.VISIBLE
            expiryDateSelectButton.visibility = View.VISIBLE

            shelfLifeTextEdit.visibility = View.INVISIBLE
            shelfLifeTextView.visibility = View.INVISIBLE

            expiryDateOverShelfLife = true
        }

        shelfLifeRadioButton.setOnClickListener {
            expiryDateTextView.visibility = View.INVISIBLE
            expiryDateSelectButton.visibility = View.INVISIBLE

            shelfLifeTextEdit.visibility = View.VISIBLE
            shelfLifeTextView.visibility = View.VISIBLE

            expiryDateOverShelfLife = false
        }

        dateOfProductionSelectButton.setOnClickListener {
            val c = Calendar.getInstance()
            val year = c.get(Calendar.YEAR)
            val month = c.get(Calendar.MONTH)
            val day = c.get(Calendar.DAY_OF_MONTH)

            val dpd = DatePickerDialog(this, { _, y, m, d ->
                dateOfProduction = "$d.${m+1}.$y"
                findViewById<TextView>(R.id.dateOfProductionTextView).text = "Date of production: $dateOfProduction"
            }, year, month, day)

            dpd.show()
        }

        expiryDateSelectButton.setOnClickListener {
            val c = Calendar.getInstance()
            val year = c.get(Calendar.YEAR)
            val month = c.get(Calendar.MONTH)
            val day = c.get(Calendar.DAY_OF_MONTH)

            val dpd = DatePickerDialog(this, { view, y, m, d ->
                dateOfExpiry = "$d.${m+1}.$y"
                expiryDateTextView.text = "Expiry date: $dateOfExpiry"
            }, year, month, day)

            dpd.show()
        }

        saveProductButton.setOnClickListener {
            if (expiryDateOverShelfLife == null) {
                Toast.makeText(this, "Please, choose and fill in shelf life or expiry date.", Toast.LENGTH_SHORT).show()
                return@setOnClickListener
            }
            if (dateOfProduction == "") {
                Toast.makeText(this, "Please, choose date of production.", Toast.LENGTH_SHORT).show()
                return@setOnClickListener
            }

            if (!expiryDateOverShelfLife!!) {
                if (shelfLifeTextEdit.text.toString() != "") {
                    evaluateDateOfExpiry(shelfLifeTextEdit.text.toString().toInt())
                }
            } else {
                if (dateOfExpiry == "") {
                    Toast.makeText(this, "Please, choose date of expiry.", Toast.LENGTH_SHORT).show()
                    return@setOnClickListener
                }
            }

            if (amountTextEdit.text.toString() == "") {
                Toast.makeText(this, "Please, specify an amount of product.", Toast.LENGTH_SHORT).show()
                return@setOnClickListener
            }

            val abstractProduct = DBStorageController(this).findAbstractProductByBarcode(DBStorageController(this).readableDatabase, barcode)
            product = Product(0, abstractProduct!!.id, amountTextEdit.text.toString().toInt(), dateOfProduction, dateOfExpiry)

            DBStorageController(this).insertNewProduct(DBStorageController(this).writableDatabase, product!!)
            finish()
        }
    }

    fun evaluateDateOfExpiry(shelfLifeMonths: Int) {
        if (dateOfProduction == "" ) {
            return
        }
        val c = Calendar.getInstance()
        val dateOfProductionSplit = dateOfProduction.split(".")
        c.set(Calendar.YEAR, dateOfProductionSplit[0].toInt())
        c.set(Calendar.MONTH, dateOfProductionSplit[1].toInt())
        c.set(Calendar.DAY_OF_MONTH, dateOfProductionSplit[2].toInt())

        c.add(Calendar.MONTH, shelfLifeMonths)
        dateOfExpiry = "${c.get(Calendar.YEAR)}.${c.get(Calendar.MONTH)}.${c.get(Calendar.DAY_OF_MONTH)}"
    }

    fun findAndDisplayAbstractProductByBarcode(barcode: String) {
        val abstractProduct = DBStorageController(this).findAbstractProductByBarcode(DBStorageController(this).readableDatabase, barcode)
        if (abstractProduct == null) {
            AlertDialog.Builder(this).setMessage("Product with such barcode not found. Try scanning with better lightning or add new abstract product")
                .setPositiveButton("Add") { _: DialogInterface, _: Int ->
                    val addAbstractProductIntent = Intent(this, AddAbstractProductActivity::class.java)
                    val extras = Bundle()
                    extras.putParcelable("abstractProduct", AbstractProduct(0, barcode, "", 0.0, "", 0))
                    addAbstractProductIntent.putExtras(extras)
                    ContextCompat.startActivity(this, addAbstractProductIntent, extras)
                }
                .setNegativeButton("Try again") {_: DialogInterface, _: Int ->

                }.show()
            return
        }
        abstractProductView.abstractProduct = abstractProduct
        abstractProductView.update()
    }
}