package org.foxarmy.barcodescannerforemployees.views

import android.app.Activity
import android.content.Context
import android.graphics.Color
import android.graphics.drawable.GradientDrawable
import android.os.Build
import android.util.AttributeSet
import android.util.Log
import android.view.LayoutInflater
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.annotation.RequiresApi
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.core.content.ContextCompat
import androidx.core.graphics.blue
import androidx.core.graphics.green
import androidx.core.graphics.red
import androidx.core.math.MathUtils.clamp
import org.foxarmy.barcodescannerforemployees.DBStorageController
import org.foxarmy.barcodescannerforemployees.R
import org.foxarmy.barcodescannerforemployees.dataclasses.AbstractProduct
import org.foxarmy.barcodescannerforemployees.dataclasses.Product
import org.foxarmy.barcodescannerforemployees.getActivity
import org.foxarmy.barcodescannerforemployees.getImageUri
import java.io.File
import java.time.Duration
import java.time.LocalDate
import java.time.format.DateTimeFormatter
import kotlin.concurrent.thread

class ProductView: LinearLayout {
    var product: Product = Product()
    var isProductSelected = false
    private lateinit var activity: Activity

    private lateinit var productImageView: ImageView
    private lateinit var productNameTextView: TextView
    private lateinit var productNetWeightTextView: TextView
    private lateinit var productAmountTextView: TextView
    private lateinit var productCategoryView: TextView
    private lateinit var productLifeSpan: TextView
    private var backgroundColor: Int = 0xffffff
    private var strokeColor: Int = 0x000000
    private lateinit var outline: GradientDrawable

    constructor(context: Context, a: AttributeSet) : super(context, a) {
        activity = getActivity(context)!!
        val inflater: LayoutInflater = activity.layoutInflater
        inflater.inflate(R.layout.product_view, this)
    }

    @RequiresApi(Build.VERSION_CODES.O)
    constructor(activity: Activity, context: Context, product: Product) : super(context) {
        this.product = product
        this.activity = activity
        val inflater: LayoutInflater = activity.layoutInflater
        inflater.inflate(R.layout.product_view, this)

        productImageView = findViewById(R.id.productPicture)
        productNameTextView = findViewById(R.id.productNameView)
        productNetWeightTextView = findViewById(R.id.productNetWeightView)
        productAmountTextView = findViewById(R.id.amountView)
        productCategoryView = findViewById(R.id.categoryView)
        productLifeSpan = findViewById(R.id.dateSpan)

        findViewById<ConstraintLayout>(R.id.productLayout).setOnLongClickListener {
            isProductSelected = !isProductSelected
            updateStroke()
            true
        }

        update()
    }

    @RequiresApi(Build.VERSION_CODES.O)
    fun update () {
        val linkedAbstractProduct: AbstractProduct = DBStorageController(activity).findAbstractProductById(DBStorageController(activity).readableDatabase, product.abstractProductId)!!

        val thumbnailsDir = File(activity.cacheDir, "thumbnails")
        thumbnailsDir.mkdirs()
        val pictureFile = File(thumbnailsDir, "${linkedAbstractProduct.imageHash}.webp")
        val imageUri = getImageUri(activity, pictureFile)
        productImageView.setImageURI(imageUri)
        productImageView.rotation = 90f

        productNameTextView.text = linkedAbstractProduct.name
        productNetWeightTextView.text = linkedAbstractProduct.netWeight.toString()
        productAmountTextView.text = product.amount.toString()
        productCategoryView.text = DBStorageController(activity).getCategoryNameById(DBStorageController(activity).readableDatabase, linkedAbstractProduct.category)
        productLifeSpan.text = "${product.dateOfProduction}-${product.dateOfExpiry}"


        updateStroke()
    }

    @RequiresApi(Build.VERSION_CODES.O)
    fun updateStroke() {
         if (isProductSelected) {
             this.background = ContextCompat.getDrawable(context, R.drawable.outline_selected)
        } else {
             if (product.id != 0) {
                 thread {
                     this.outline = GradientDrawable()
                     backgroundColor = evaluateColor()
                     strokeColor = darkenColor(backgroundColor, 0.25) // (backgroundColor and 0xfefefe ) shr 1
                     this.outline.setColor(backgroundColor)
                     this.outline.setStroke(4, strokeColor)
                     this.outline.alpha = 84
                     this.background = outline
                 }
             } else {
                 this.background = ContextCompat.getDrawable(context, R.drawable.outline)
             }
        }
    }

    @RequiresApi(Build.VERSION_CODES.O)
    fun evaluateColor(): Int {
        val dateFormatter: DateTimeFormatter =  DateTimeFormatter.ofPattern("d.M.yyyy")

        val fresh = LocalDate.parse(product.dateOfProduction, dateFormatter)
        val expired = LocalDate.parse(product.dateOfExpiry, dateFormatter)

        val shelfLife = Duration.between(fresh.atStartOfDay(), expired.atStartOfDay()).toDays()
        val today = LocalDate.parse(LocalDate.now().format(dateFormatter), dateFormatter)
        val daysBeforeExpiry = Duration.between(today.atStartOfDay(), expired.atStartOfDay()).toDays()

        val freshnessPercentage: Double = daysBeforeExpiry / shelfLife.toDouble()

        return calculateFreshnessGradient(freshnessPercentage)
    }

    fun calculateFreshnessGradient(percentage: Double): Int {
        val startColor = ContextCompat.getColor(context, if (percentage > 0.5) R.color.full_freshness else R.color.half_freshness)
        val endColor = ContextCompat.getColor(context, if (percentage > 0.5) R.color.half_freshness else R.color.expired_freshness)

        val gradientPosition = 1 - if (percentage > 0.5) (percentage - 0.5) * 2 else percentage * 2

        val red = clamp((startColor.red + gradientPosition * (endColor.red - startColor.red )).toInt(), 0, 255)
        val green = clamp((startColor.green  + gradientPosition * (endColor.green - startColor.green)).toInt(), 0, 255)
        val blue = clamp((startColor.blue + gradientPosition * (endColor.blue - startColor.blue)).toInt(), 0, 255)

        var redHex = Integer.toHexString(red)
        if (redHex.length == 1) redHex = "0$redHex"

        var greenHex = Integer.toHexString(green)
        if (greenHex.length == 1) greenHex = "0$greenHex"

        var blueHex = Integer.toHexString(blue)
        if (blueHex.length == 1 ) blueHex = "0$blueHex"


        val colorString = "#$redHex$greenHex$blueHex"
        Log.d("QWERTYUIOP", "$red:$green:$blue")
        Log.d("QWERTYUIOP", "$redHex-$greenHex-$blueHex")
        Log.d("QWERTYUIOP", colorString)
        return Color.parseColor(colorString)
    }

    @RequiresApi(Build.VERSION_CODES.O)
    fun darkenColor(color: Int, darkPercent: Double) : Int {
        val c = Color.valueOf(color)

        val red = c.red() * (1 - darkPercent)
        val green = c.green() * (1 - darkPercent)
        val blue = c.blue() * (1 - darkPercent)
        Log.d("QWERTYUIOP", "....A:${c.red()}, ${c.green()}, ${c.blue()}")
        Log.d("QWERTYUIOP", "....B:${red.toFloat()}, ${green.toFloat()}, ${blue.toFloat()}")

        return Color.rgb(red.toFloat(), green.toFloat(), blue.toFloat())
    }
}